@@ -4,8 +4,10 @@ Tests for bitarray.util module
 from __future__ import absolute_import
 
 import os
+import re
 import sys
 import base64
+import binascii
 import shutil
 import tempfile
 import unittest
@@ -21,7 +23,7 @@ from bitarray.util import (
     zeros, urandom, pprint, make_endian, rindex, strip, count_n,
     parity, count_and, count_or, count_xor, subset,
     serialize, deserialize, ba2hex, hex2ba, ba2base, base2ba,
-    ba2int, int2ba, huffman_code
+    ba2int, int2ba, vl_encode, vl_decode, huffman_code
 )
 
 if sys.version_info[0] == 3:
@@ -29,24 +31,28 @@ if sys.version_info[0] == 3:
 else:
     from io import BytesIO as StringIO
 
-tests = []
+tests = []  # type: list
 
 # ---------------------------------------------------------------------------
 
 class TestsZeros(unittest.TestCase):
 
-    def test_1(self):
+    def test_basic(self):
         for default_endian in 'big', 'little':
             _set_default_endian(default_endian)
             a = zeros(0)
             self.assertEqual(a, bitarray())
             self.assertEqual(a.endian(), default_endian)
 
-            b  = zeros(0, endian=None)
-            self.assertEqual(b.endian(), default_endian)
+            a = zeros(0, endian=None)
+            self.assertEqual(len(a), 0)
+            self.assertEqual(a.endian(), default_endian)
 
             for n in range(100):
                 a = zeros(n)
+                self.assertEqual(len(a), n)
+                self.assertFalse(a.any())
+                self.assertEqual(a.count(1), 0)
                 self.assertEqual(a, bitarray(n * '0'))
 
             for endian in 'big', 'little':
@@ -72,28 +78,35 @@ tests.append(TestsZeros)
 
 # ---------------------------------------------------------------------------
 
-class TestsRandom(unittest.TestCase):
+class TestsURandom(unittest.TestCase):
 
-    def test_1(self):
+    def test_basic(self):
         for default_endian in 'big', 'little':
             _set_default_endian(default_endian)
             a = urandom(0)
             self.assertEqual(a, bitarray())
             self.assertEqual(a.endian(), default_endian)
 
-            b  = urandom(0, endian=None)
-            self.assertEqual(b.endian(), default_endian)
+            a = urandom(7, endian=None)
+            self.assertEqual(len(a), 7)
+            self.assertEqual(a.endian(), default_endian)
 
-            for n in range(100):
+            for n in range(50):
                 a = urandom(n)
                 self.assertEqual(len(a), n)
-                self.assertEqual(b.endian(), default_endian)
+                self.assertEqual(a.endian(), default_endian)
 
-            a = urandom(1000)
-            b = urandom(1000)
-            self.assertNotEqual(a, b)
-            self.assertTrue(400 < a.count() < 600)
-            self.assertTrue(400 < b.count() < 600)
+            for endian in 'big', 'little':
+                a = urandom(11, endian)
+                self.assertEqual(len(a), 11)
+                self.assertEqual(a.endian(), endian)
+
+    def test_count(self):
+        a = urandom(1000)
+        b = urandom(1000)
+        self.assertNotEqual(a, b)
+        self.assertTrue(400 < a.count() < 600)
+        self.assertTrue(400 < b.count() < 600)
 
     def test_wrong_args(self):
         self.assertRaises(TypeError, urandom)
@@ -106,7 +119,7 @@ class TestsRandom(unittest.TestCase):
         self.assertRaises(TypeError, urandom, 0, 1)
         self.assertRaises(ValueError, urandom, 0, 'foo')
 
-tests.append(TestsRandom)
+tests.append(TestsURandom)
 
 # ---------------------------------------------------------------------------
 
@@ -189,7 +202,7 @@ class TestsMakeEndian(unittest.TestCase, Util):
         a = bitarray('1110001', endian='big')
         b = make_endian(a, 'big')
         self.assertTrue(b is a)
-        c = make_endian(a, 'little')
+        c = make_endian(a, endian='little')
         self.assertTrue(c == a)
         self.assertEqual(c.endian(), 'little')
         self.assertIsType(c, 'bitarray')
@@ -235,23 +248,33 @@ class TestsRIndex(unittest.TestCase, Util):
     def test_simple(self):
         self.assertRaises(TypeError, rindex)
         self.assertRaises(TypeError, rindex, None)
-        self.assertRaises(TypeError, rindex, bitarray(), 1, 2)
+        self.assertRaises(ValueError, rindex, bitarray(), 1)
         for endian in 'big', 'little':
-            a = bitarray('00010110000', endian)
+            a = bitarray('00010110 000', endian)
             self.assertEqual(rindex(a), 6)
             self.assertEqual(rindex(a, 1), 6)
-            self.assertEqual(rindex(a, 'A'), 6)
-            self.assertEqual(rindex(a, True), 6)
+            self.assertEqual(rindex(a, 1, 3), 6)
+            self.assertEqual(rindex(a, 1, 3, 8), 6)
+            self.assertEqual(rindex(a, 1, -20, 20), 6)
+            self.assertEqual(rindex(a, 1, 0, 5), 3)
+            self.assertEqual(rindex(a, 1, 0, -6), 3)
+            self.assertEqual(rindex(a, 1, 0, -5), 5)
+            self.assertRaises(TypeError, rindex, a, 'A')
+            self.assertRaises(ValueError, rindex, a, 2)
+            self.assertRaises(ValueError, rindex, a, 1, 7)
+            self.assertRaises(ValueError, rindex, a, 1, 10, 3)
+            self.assertRaises(ValueError, rindex, a, 1, -1, 0)
+            self.assertRaises(TypeError, rindex, a, 1, 10, 3, 4)
 
-            a = bitarray('00010110111', endian)
+            a = bitarray('00010110 111', endian)
             self.assertEqual(rindex(a, 0), 7)
-            self.assertEqual(rindex(a, None), 7)
+            self.assertEqual(rindex(a, 0, 0, 4), 2)
             self.assertEqual(rindex(a, False), 7)
 
-            a = frozenbitarray('00010110111', endian)
+            a = frozenbitarray('00010110 111', endian)
             self.assertEqual(rindex(a, 0), 7)
-            self.assertEqual(rindex(a, None), 7)
-            self.assertEqual(rindex(a, False), 7)
+            self.assertRaises(TypeError, rindex, a, None)
+            self.assertRaises(ValueError, rindex, a, 7)
 
             for v in 0, 1:
                 self.assertRaises(ValueError, rindex,
@@ -261,6 +284,20 @@ class TestsRIndex(unittest.TestCase, Util):
             self.assertRaises(ValueError, rindex,
                               bitarray('11111', endian), 0)
 
+    def test_range(self):
+        n = 250
+        a = bitarray(n)
+        for m in range(n):
+            a.setall(0)
+            self.assertRaises(ValueError, rindex, a, 1)
+            a[m] = 1
+            self.assertEqual(rindex(a, 1), m)
+
+            a.setall(1)
+            self.assertRaises(ValueError, rindex, a, 0)
+            a[m] = 0
+            self.assertEqual(rindex(a, 0), m)
+
     def test_random(self):
         for a in self.randombitarrays():
             v = randint(0, 1)
@@ -275,9 +312,26 @@ class TestsRIndex(unittest.TestCase, Util):
                 j = None
             self.assertEqual(i, j)
 
+    def test_random_start_stop(self):
+        n = 2000
+        a = zeros(n)
+        indices = [randint(0, n - 1) for _ in range(100)]
+        for i in indices:
+            a[i] = 1
+        for _ in range(100):
+            start = randint(0, n)
+            stop = randint(0, n)
+            filtered = [i for i in indices if i >= start and i < stop]
+            ref = max(filtered) if filtered else -1
+            try:
+                res = rindex(a, 1, start, stop)
+            except ValueError:
+                res = -1
+            self.assertEqual(res, ref)
+
     def test_many_set(self):
         for _ in range(10):
-            n = randint(1, 100000)
+            n = randint(1, 10000)
             v = randint(0, 1)
             a = bitarray(n)
             a.setall(not v)
@@ -366,27 +420,50 @@ class TestsCount_N(unittest.TestCase, Util):
 
     @staticmethod
     def count_n(a, n):
-        "return the index i for which a[:i].count() == n"
+        "return lowest index i for which a[:i].count() == n"
         i, j = n, a.count(1, 0, n)
         while j < n:
-            if a[i]:
-                j += 1
+            j += a[i]
             i += 1
         return i
 
-    def check_result(self, a, n, i):
-        self.assertEqual(a.count(1, 0, i), n)
-        if i > 0:
-            self.assertTrue(a[i - 1])
+    def check_result(self, a, n, i, v=1):
+        self.assertEqual(a.count(v, 0, i), n)
+        if i == 0:
+            self.assertEqual(n, 0)
+        else:
+            self.assertEqual(a[i - 1], v)
+
+    def test_empty(self):
+        a = bitarray()
+        self.assertEqual(count_n(a, 0), 0)
+        self.assertEqual(count_n(a, 0, 0), 0)
+        self.assertEqual(count_n(a, 0, 1), 0)
+        self.assertRaises(ValueError, count_n, a, 1)
+        self.assertRaises(TypeError, count_n, '', 0)
+        self.assertRaises(TypeError, count_n, a, 7.0)
+        self.assertRaises(ValueError, count_n, a, 0, 2)
 
     def test_simple(self):
         a = bitarray('111110111110111110111110011110111110111110111000')
         b = a.copy()
         self.assertEqual(len(a), 48)
         self.assertEqual(a.count(), 37)
-        self.assertRaises(TypeError, count_n, '', 0)
+        self.assertEqual(a.count(0), 11)
+
+        self.assertEqual(count_n(a, 0, 0), 0)
+        self.assertEqual(count_n(a, 2, 0), 12)
+        self.assertEqual(count_n(a, 10, 0), 47)
+        self.assertRaisesMessage(ValueError, "non-negative integer expected",
+                                 count_n, a, -1, 0) # n < 0
+        self.assertRaisesMessage(ValueError, "n larger than bitarray size",
+                                 count_n, a, 49, 0) # n > len(a)
+        self.assertRaisesMessage(ValueError, "n exceeds total count",
+                                 count_n, a, 12, 0) # n > a.count(0)
+
         self.assertEqual(count_n(a, 0), 0)
         self.assertEqual(count_n(a, 20), 23)
+        self.assertEqual(count_n(a, 20, 1), 23)
         self.assertEqual(count_n(a, 37), 45)
         self.assertRaisesMessage(ValueError, "non-negative integer expected",
                                  count_n, a, -1) # n < 0
@@ -394,58 +471,75 @@ class TestsCount_N(unittest.TestCase, Util):
                                  count_n, a, 49) # n > len(a)
         self.assertRaisesMessage(ValueError, "n exceeds total count",
                                  count_n, a, 38) # n > a.count()
-        self.assertRaises(TypeError, count_n, a, "7")
-        for n in range(0, 37):
-            i = count_n(a, n)
-            self.check_result(a, n, i)
-            self.assertEqual(a[:i].count(), n)
-            self.assertEqual(i, self.count_n(a, n))
+
+        for v in 0, 1:
+            for n in range(a.count(v) + 1):
+                i = count_n(a, n, v)
+                self.check_result(a, n, i, v)
+                self.assertEqual(a[:i].count(v), n)
+                self.assertEqual(i, self.count_n(a if v else ~a, n))
         self.assertEQUAL(a, b)
 
     def test_frozen(self):
         a = frozenbitarray('001111101111101111101111100111100')
         self.assertEqual(len(a), 33)
         self.assertEqual(a.count(), 24)
-        self.assertRaises(TypeError, count_n, '', 0)
         self.assertEqual(count_n(a, 0), 0)
         self.assertEqual(count_n(a, 10), 13)
         self.assertEqual(count_n(a, 24), 31)
         self.assertRaises(ValueError, count_n, a, -1) # n < 0
         self.assertRaises(ValueError, count_n, a, 25) # n > a.count()
         self.assertRaises(ValueError, count_n, a, 34) # n > len(a)
-        self.assertRaises(TypeError, count_n, a, "7")
-
-    def test_large(self):
-        for N in list(range(100)) + [1000, 10000, 100000]:
-            a = bitarray(N)
-            v = randint(0, 1)
-            a.setall(not v)
-            for _ in range(randint(0, min(N, 100))):
-                a[randint(0, N - 1)] = v
-            n = randint(0, a.count())
+        for n in range(25):
             self.check_result(a, n, count_n(a, n))
-            # check for total count
-            tc = a.count()
-            self.assertTrue(count_n(a, tc) <= N)
-            self.assertRaises(ValueError, count_n, a, tc + 1)
+
+    def test_ones(self):
+        n = randint(1, 100000)
+        a = bitarray(n)
+        a.setall(1)
+        self.assertEqual(count_n(a, n), n)
+        self.assertRaises(ValueError, count_n, a, 1, 0)
+        self.assertRaises(ValueError, count_n, a, n + 1)
+        for _ in range(20):
+            i = randint(0, n)
+            self.assertEqual(count_n(a, i), i)
 
     def test_one_set(self):
-        N = 100000
-        for _ in range(10):
-            a = bitarray(N)
+        n = randint(1, 100000)
+        a = bitarray(n)
+        a.setall(0)
+        self.assertEqual(count_n(a, 0), 0)
+        self.assertRaises(ValueError, count_n, a, 1)
+        for _ in range(20):
             a.setall(0)
-            self.assertEqual(count_n(a, 0), 0)
-            self.assertRaises(ValueError, count_n, a, 1)
-            i = randint(0, N - 1)
+            i = randint(0, n - 1)
             a[i] = 1
             self.assertEqual(count_n(a, 1), i + 1)
             self.assertRaises(ValueError, count_n, a, 2)
 
+    def test_large(self):
+        for _ in range(100):
+            N = randint(100000, 250000)
+            a = bitarray(N)
+            v = randint(0, 1)
+            a.setall(not v)
+            for _ in range(randint(0, 100)):
+                a[randint(0, N - 1)] = v
+            tc = a.count(v)      # total count
+            i = count_n(a, tc, v)
+            self.check_result(a, tc, i, v)
+            self.assertRaises(ValueError, count_n, a, tc + 1, v)
+            for _ in range(20):
+                n = randint(0, tc)
+                i = count_n(a, n, v)
+                self.check_result(a, n, i, v)
+
     def test_random(self):
         for a in self.randombitarrays():
-            n = a.count() // 2
-            i = count_n(a, n)
-            self.check_result(a, n, i)
+            for v in 0, 1:
+                n = a.count(v) // 2
+                i = count_n(a, n, v)
+                self.check_result(a, n, i, v)
 
 tests.append(TestsCount_N)
 
@@ -458,7 +552,7 @@ class TestsBitwiseCount(unittest.TestCase, Util):
         ones.setall(1)
         zeros = bitarray(8)
         zeros.setall(0)
-        for i in range(0, 256):
+        for i in range(256):
             a = bitarray()
             a.frombytes(bytes(bytearray([i])))
             cnt = a.count()
@@ -570,10 +664,10 @@ class TestsParity(unittest.TestCase, Util):
 
     def test_bitarray(self):
         a = bitarray()
-        self.assertTrue(parity(a) is False)
+        self.assertBitEqual(parity(a), 0)
         par = False
         for _ in range(100):
-            self.assertTrue(parity(a) is par)
+            self.assertEqual(parity(a), par)
             a.append(1)
             par = not par
 
@@ -582,14 +676,10 @@ class TestsParity(unittest.TestCase, Util):
         a.setall(1)
         self.assertTrue(parity(a))
 
-    def test_return_type(self):
-        p = parity(bitarray('101'))
-        self.assertIsInstance(p, bool)
-
     def test_frozenbitarray(self):
-        self.assertTrue(parity(frozenbitarray()) is False)
-        self.assertTrue(parity(frozenbitarray('0010011')) is True)
-        self.assertTrue(parity(frozenbitarray('10100110')) is False)
+        self.assertBitEqual(parity(frozenbitarray()), 0)
+        self.assertBitEqual(parity(frozenbitarray('0010011')), 1)
+        self.assertBitEqual(parity(frozenbitarray('10100110')), 0)
 
     def test_wrong_args(self):
         self.assertRaises(TypeError, parity, '')
@@ -609,20 +699,6 @@ tests.append(TestsParity)
 
 # ---------------------------------------------------------------------------
 
-CODEDICT = {'little': {}, 'big': {
-    '0': bitarray('0000'),    '1': bitarray('0001'),
-    '2': bitarray('0010'),    '3': bitarray('0011'),
-    '4': bitarray('0100'),    '5': bitarray('0101'),
-    '6': bitarray('0110'),    '7': bitarray('0111'),
-    '8': bitarray('1000'),    '9': bitarray('1001'),
-    'a': bitarray('1010'),    'b': bitarray('1011'),
-    'c': bitarray('1100'),    'd': bitarray('1101'),
-    'e': bitarray('1110'),    'f': bitarray('1111'),
-}}
-for k, v in CODEDICT['big'].items():
-    CODEDICT['little'][k] = v[::-1]
-
-
 class TestsHexlify(unittest.TestCase, Util):
 
     def test_ba2hex(self):
@@ -675,16 +751,6 @@ class TestsHexlify(unittest.TestCase, Util):
             for b in b'\0', b'\0f', b'f\0', b'\0ff', b'f\0f', b'ff\0':
                 self.assertRaises(ValueError, hex2ba, b)
 
-    @staticmethod
-    def hex2ba(s, endian=None):
-        a = bitarray(0, endian or get_default_endian())
-        a.encode(CODEDICT[a.endian()], s)
-        return a
-
-    @staticmethod
-    def ba2hex(a):
-        return ''.join(a.iterdecode(CODEDICT[a.endian()]))
-
     def test_explicit(self):
         data = [ #                       little   big
             ('',                         '',      ''),
@@ -701,14 +767,9 @@ class TestsHexlify(unittest.TestCase, Util):
             self.assertEQUAL(hex2ba(hex_le, 'little'), a_le)
             self.assertEqual(ba2hex(a_be), hex_be)
             self.assertEqual(ba2hex(a_le), hex_le)
-            # test simple encode / decode implementation
-            self.assertEQUAL(self.hex2ba(hex_be, 'big'), a_be)
-            self.assertEQUAL(self.hex2ba(hex_le, 'little'), a_le)
-            self.assertEqual(self.ba2hex(a_be), hex_be)
-            self.assertEqual(self.ba2hex(a_le), hex_le)
 
     def test_round_trip(self):
-        s = ''.join(choice(hexdigits) for _ in range(randint(10, 100)))
+        s = ''.join(choice(hexdigits) for _ in range(randint(20, 100)))
         for default_endian in 'big', 'little':
             _set_default_endian(default_endian)
             a = hex2ba(s)
@@ -719,9 +780,14 @@ class TestsHexlify(unittest.TestCase, Util):
             self.assertEqual(t, s.lower())
             b = hex2ba(t, default_endian)
             self.assertEQUAL(a, b)
-            # test simple encode / decode implementation
-            self.assertEQUAL(a, self.hex2ba(t))
-            self.assertEqual(t, self.ba2hex(a))
+
+    def test_binascii(self):
+        a = urandom(800, 'big')
+        s = binascii.hexlify(a.tobytes()).decode()
+        self.assertEqual(ba2hex(a), s)
+        b = bitarray(endian='big')
+        b.frombytes(binascii.unhexlify(s))
+        self.assertEQUAL(hex2ba(s, 'big'), b)
 
 tests.append(TestsHexlify)
 
@@ -776,12 +842,17 @@ class TestsBase(unittest.TestCase, Util):
         a = bitarray()
         self.assertRaises(TypeError, ba2base, None, a)
         self.assertRaises(TypeError, base2ba, None, '')
+        self.assertRaises(TypeError, ba2base, 16.0, a)
+        self.assertRaises(TypeError, base2ba, 16.0, '')
         for i in range(-10, 260):
             if i in (2, 4, 8, 16, 32, 64):
                 continue
             self.assertRaises(ValueError, ba2base, i, a)
             self.assertRaises(ValueError, base2ba, i, '')
 
+        self.assertRaises(TypeError, ba2base, 32, None)
+        self.assertRaises(TypeError, base2ba, 32, None)
+
     def test_binary(self):
         a = base2ba(2, '1011')
         self.assertEqual(a, bitarray('1011'))
@@ -807,7 +878,7 @@ class TestsBase(unittest.TestCase, Util):
         self.assertEqual(a, bitarray('1111 0110 0001'))
         self.assertEqual(ba2base(16, a), 'f61')
 
-        for n in range(100):
+        for n in range(50):
             s = ''.join(choice(hexdigits) for _ in range(n))
             for endian in 'big', 'little':
                 a = base2ba(16, s, endian)
@@ -878,6 +949,150 @@ tests.append(TestsBase)
 
 # ---------------------------------------------------------------------------
 
+class VLFTests(unittest.TestCase, Util):
+
+    def test_explicit(self):
+        for s, bits in [
+                (b'\x40', ''),
+                (b'\x30', '0'),
+                (b'\x38', '1'),
+                (b'\x00', '0000'),
+                (b'\x01', '0001'),
+                (b'\xe0\x40', '0000 1'),
+                (b'\x90\x02', '0000 000001'),
+                (b'\xb5\xa7\x18', '0101 0100111 0011'),
+        ]:
+            a = bitarray(bits)
+            self.assertEqual(vl_encode(a), s)
+            self.assertEqual(vl_decode(s), a)
+
+    def test_encode(self):
+        for endian in 'big', 'little':
+            s = vl_encode(bitarray('001101', endian))
+            self.assertIsInstance(s, bytes)
+            self.assertEqual(s, b'\xd3\x20')
+
+    def test_decode_args(self):
+        if sys.version_info[0] == 3:
+            self.assertRaises(TypeError, vl_decode, 'foo')
+            self.assertRaises(TypeError, vl_decode, iter([b'\x40']))
+
+        self.assertRaises(TypeError, vl_decode, b'\x40', 'big', 3)
+        self.assertRaises(ValueError, vl_decode, b'\x40', 'foo')
+        for item in None, 2.34, Ellipsis:
+            self.assertRaises(TypeError, vl_decode, iter([item]))
+
+        lst = [b'\xd3\x20', iter(b'\xd3\x20')]
+        if sys.version_info[0] == 3:
+            lst.append(iter([0xd3, 0x20]))
+        for s in lst:
+            a = vl_decode(s, endian=self.random_endian())
+            self.assertIsInstance(a, bitarray)
+            self.assertEqual(a, bitarray('0011 01'))
+
+    def test_decode_endian(self):
+        for endian in 'little', 'big', None:
+            a = vl_decode(b'\xd3\x20', endian)
+            self.assertEqual(a, bitarray('0011 01'))
+            self.assertEqual(a.endian(),
+                             endian if endian else get_default_endian())
+
+    def test_decode_trailing(self):
+        for s, bits in [(b'\x40ABC', ''),
+                        (b'\xe0\x40A', '00001')]:
+            stream = iter(s)
+            self.assertEqual(vl_decode(stream), bitarray(bits))
+            self.assertEqual(next(stream),
+                             b'A' if sys.version_info[0] == 2 else 65)
+
+    def test_decode_ambiguity(self):
+        for s in b'\x40', b'\x4f', b'\x45':
+            self.assertEqual(vl_decode(iter(s)), bitarray())
+        for s in b'\x1e', b'\x1f':
+            self.assertEqual(vl_decode(iter(s)), bitarray('111'))
+
+    def test_decode_stream(self):
+        stream = iter(b'\x40\x30\x38\x40\x2c\xe0\x40\xd3\x20')
+        for bits in '', '0', '1', '', '11', '0000 1', '0011 01':
+            self.assertEqual(vl_decode(stream), bitarray(bits))
+
+        arrays = [urandom(randint(0, 30)) for _ in range(1000)]
+        stream = iter(b''.join(vl_encode(a) for a in arrays))
+        for a in arrays:
+            self.assertEqual(vl_decode(stream), a)
+
+    def test_decode_errors(self):
+        # decode empty bits
+        self.assertRaises(StopIteration, vl_decode, b'')
+        # invalid number of padding bits
+        for s in b'\x50', b'\x60', b'\x70':
+            self.assertRaises(ValueError, vl_decode, s)
+        self.assertRaises(ValueError, vl_decode, b'\xf0')
+        # high bit set, but no terminating byte
+        for s in b'\x80', b'\x80\x80':
+            self.assertRaises(StopIteration, vl_decode, s)
+
+    def test_decode_error_message(self):
+        pat = re.compile(r'[\w\s,]+:\s+(\d+)')
+        for n in range(120):
+            a = None
+            s = bytes(bytearray([randint(0x80, 0xef) for _ in range(n)]))
+            try:
+                a = vl_decode(s)
+            except StopIteration as e:
+                m = pat.match(str(e))
+                self.assertEqual(m.group(1), str(n))
+            self.assertTrue(a is None)
+
+    def test_decode_invalid_stream(self):
+        if sys.version_info[0] == 2:
+            return
+        N = 100
+        s = iter(N * (3 * [0x80] + ['XX']) + ['end.'])
+        for _ in range(N):
+            a = None
+            try:
+                a = vl_decode(s)
+            except TypeError:
+                pass
+            self.assertTrue(a is None)
+        self.assertEqual(next(s), 'end.')
+
+    def test_explicit_zeros(self):
+        for n in range(100):
+            a = zeros(4 + n * 7)
+            s = n * b'\x80' + b'\x00'
+            self.assertEqual(vl_encode(a), s)
+            self.assertEqual(vl_decode(s), a)
+
+    def round_trip(self, a):
+        s = vl_encode(a)
+        b = vl_decode(s)
+        self.check_obj(b)
+        self.assertEqual(a, b)
+        PADBITS = 3
+        self.assertEqual(len(s), (len(a) + PADBITS + 6) // 7)
+
+        head = ord(s[0]) if sys.version_info[0] == 2 else s[0]
+        padding = (head & 0x70) >> 4
+        self.assertEqual(len(a) + padding, 7 * len(s) - PADBITS)
+
+    def test_range(self):
+        for n in range(500):
+            self.round_trip(urandom(n))
+
+    def test_large(self):
+        a = urandom(randint(50000, 100000))
+        self.round_trip(a)
+
+    def test_random(self):
+        for a in self.randombitarrays():
+            self.round_trip(a)
+
+tests.append(VLFTests)
+
+# ---------------------------------------------------------------------------
+
 class TestsIntegerization(unittest.TestCase, Util):
 
     def test_ba2int(self):
@@ -895,6 +1110,28 @@ class TestsIntegerization(unittest.TestCase, Util):
         # ensure original object wasn't altered
         self.assertEQUAL(a, b)
 
+    def test_ba2int_frozen(self):
+        for a in self.randombitarrays(start=1):
+            b = frozenbitarray(a)
+            self.assertEqual(ba2int(b), ba2int(a))
+            self.assertEQUAL(a, b)
+
+    def test_ba2int_random(self):
+        for a in self.randombitarrays(start=1):
+            b = bitarray(a, 'big')
+            self.assertEqual(a, b)
+            self.assertEqual(ba2int(b), int(b.to01(), 2))
+
+    def test_ba2int_bytes(self):
+        for n in range(1, 50):
+            a = urandom(8 * n, self.random_endian())
+            c = bytearray(a.tobytes())
+            i = 0
+            for x in (c if a.endian() == 'big' else reversed(c)):
+                i <<= 8
+                i |= x
+            self.assertEqual(ba2int(a), i)
+
     def test_int2ba(self):
         self.assertEqual(int2ba(0), bitarray('0'))
         self.assertEqual(int2ba(1), bitarray('1'))
@@ -930,24 +1167,25 @@ class TestsIntegerization(unittest.TestCase, Util):
                 ('11110',  15),
                 ('00001', -16),
                 ('11111',  -1),
-                ('000000000',    0),
-                ('111111110',  255),
-                ('000000001', -256),
-                ('111111111',   -1),
-                ('0000000000000000000000', 0),
-                ('1001000011000000100010', 9 + 3 * 256 + 17 * 2 ** 16),
-                ('1111111111111111111110', 2 ** 21 - 1),
-                ('0000000000000000000001', -2 ** 21),
-                ('1001000011000000100011', -2 ** 21
+                ('00000000 0',    0),
+                ('11111111 0',  255),
+                ('00000000 1', -256),
+                ('11111111 1',   -1),
+                ('00000000 00000000 000000', 0),
+                ('10010000 11000000 100010', 9 + 3 * 256 + 17 * 2 ** 16),
+                ('11111111 11111111 111110', 2 ** 21 - 1),
+                ('00000000 00000000 000001', -2 ** 21),
+                ('10010000 11000000 100011', -2 ** 21
                                            + (9 + 3 * 256 + 17 * 2 ** 16)),
-                ('1111111111111111111111', -1),
+                ('11111111 11111111 111111', -1),
         ]:
             self.assertEqual(ba2int(bitarray(s, 'little'), signed=1), i)
             self.assertEqual(ba2int(bitarray(s[::-1], 'big'), signed=1), i)
 
-            self.assertEQUAL(int2ba(i, len(s), 'little', signed=1),
+            len_s = len(bitarray(s))
+            self.assertEQUAL(int2ba(i, len_s, 'little', signed=1),
                              bitarray(s, 'little'))
-            self.assertEQUAL(int2ba(i, len(s), 'big', signed=1),
+            self.assertEQUAL(int2ba(i, len_s, 'big', signed=1),
                              bitarray(s[::-1], 'big'))
 
     def test_int2ba_overflow(self):
@@ -1102,7 +1340,7 @@ class MixedTests(unittest.TestCase, Util):
             n = randint(0, len(a))
             if a.endian() == 'big':
                 self.assertEqual(ba2int(a >> n), i >> n)
-                c = zeros(len(a)) + a
+                c = zeros(len(a), 'big') + a
                 self.assertEqual(ba2int(c << n), i << n)
 
             self.assertEQUAL(a, aa)
@@ -1130,10 +1368,27 @@ class MixedTests(unittest.TestCase, Util):
                 c = a.copy()
                 c >>= n
                 self.assertEqual(ba2int(c), i >> n)
-                c = zeros(len(a)) + a
+                c = zeros(len(a), 'big') + a
                 c <<= n
                 self.assertEqual(ba2int(c), i << n)
 
+    def test_primes(self):  # Sieve of Eratosthenes
+        sieve = bitarray(10000)
+        sieve.setall(1)
+        sieve[:2] = 0  # zero and one are not prime
+        for i in range(2, 100):
+            if sieve[i]:
+                sieve[i * i::i] = 0
+        # the first 15 primes
+        self.assertEqual(sieve.search(1, 15), [2, 3, 5, 7, 11, 13, 17, 19,
+                                               23, 29, 31, 37, 41, 43, 47])
+        # there are 1229 primes between 1 and 10000
+        self.assertEqual(sieve.count(1), 1229)
+        # there are 119 primes between 4000 and 5000
+        self.assertEqual(sieve.count(1, 4000, 5000), 119)
+        # the 1000th prime is 7919
+        self.assertEqual(count_n(sieve, 1000) - 1, 7919)
+
 tests.append(MixedTests)
 
 # ---------------------------------------------------------------------------
@@ -1157,6 +1412,7 @@ class TestsSerialization(unittest.TestCase, Util):
             for n in range(100):
                 a = zeros(n, endian)
                 s = serialize(a)
+                self.assertIsInstance(s, bytes)
                 self.assertEqual(s[1:], b'\0' * bits2bytes(n))
                 self.assertEQUAL(a, deserialize(s))
                 a.setall(1)
@@ -1185,7 +1441,7 @@ class TestsSerialization(unittest.TestCase, Util):
                 self.assertRaises(ValueError, deserialize, b)
 
     def test_bits_ignored(self):
-        # the unused bits (with the last bytes) are ignored
+        # the unused padding bits (with the last bytes) are ignored
         for b, a in [
                 (b'\x07\x01', bitarray('1', 'little')),
                 (b'\x07\x03', bitarray('1', 'little')),
@@ -1297,10 +1553,9 @@ tests.append(TestsHuffman)
 # ---------------------------------------------------------------------------
 
 def run(verbosity=1):
-    import os
     import bitarray
 
-    print('bitarray is installed in: %s' % os.path.dirname(bitarray.__file__))
+    print('bitarray.util is installed in: %s' % os.path.dirname(__file__))
     print('bitarray version: %s' % bitarray.__version__)
     print('Python version: %s' % sys.version)
 
